# Theme Development

Themes are a collection of page templates, assets (CSS, JS, etc) and a metadata file.  They should use responsive design and be easily edited and viewed.  Providing a variety of color schemes is recommended for users to choose from when building their site.

## Folder Structure

This is the folder structure for a standard theme.  The root folder shown below should be placed in `[sitewriter app]/public/themes`.  **Bold** denotes a folder, *italic* denotes required.

* Theme Name
    * _theme.json_
    * contact.php
    * _default.php_
    * _404.php_
    * ***colors***
        * ***default***
        * **green**
        * **etc**
                

## Metadata File

The metadata file is required for SiteWriter to recognize the theme.  It must be saved as `theme.json` in the root folder of the theme.

It contains the theme name, singlepage flag, list of page templates, and list of theme colors.

`singlepage` is a flag that tells SiteWriter if the theme is designed for a single-page website.  If it is set to true, page creation will be disabled and the site will only have a single page.

`templates` is a list of page templates.  A template with an ID of `default` is required.  The template ID is the name of the template file without the .php extension.  For example, SiteWriter loads `home.php` to show a page with the `home` template.

`colors` is a list of color schemes for the theme.  The color ID is the name of the folder where the color-specific assets are stored.  A color of `default` is required.  If the chosen color is `green`, the color-specific assets should be placed in `colors/green` in the theme folder.

### Sample `theme.json`

```js
{
    "name": "My Theme",
    "singlepage": false,
    "templates": {
        "default": {
            "title": "Default",
            "description": "A regular page."
        },
        "home": {
            "title": "Home",
            "description": "A homepage."
        },
        "contact": {
            "title": "Contact",
            "description": "A contact page."
        },
    },
    "colors": {
        "default": {
            "title": "Default",
            "description": "Blue color theme"
        },
        "green": {
            "title": "Green",
            "description": "Green color theme"
        }
    }
}

```

## Templates

### `default.php`

`default.php` is the default template file for a normal page in your theme.  It is a standard PHP script containing HTML markup.  It should never be loaded directly by a web browser, so you can safely assume theme functions and other SiteWriter theme utilities to be present.

### `404.php`

As the name implies, this template is loaded to handle 404 errors.  It is not user-selectable as a page template.  It should not contain any user-editable fields, as there is currently no way for a user to edit site error pages.

### `contact.php`

It is highly recommended to have a contact form template included in your theme.  It should contain a standard form that POSTs to `contact.php` with the fields `name`, `email`, and `message`.

## Template Requirements and Considerations

Do not use the optional `<html>`, `<head>`, and `<body>` tags for your template HTML.  The editor relies on being able to append code to the document and have it still work normally.

Do include FontAwesome 5 in your theme.  The SiteWriter editor allows the user to pick icons from the FontAwesome Free icon set to enhance their content.

Do include company info (business name, phone number, address, and email) and social media links (Facebook, Twitter, YouTube, Mastodon, etc) in the theme.  These can be setup in the sitewide settings, and users will expect themes to make use of this information.

Do include `<?php get_header(); ?>` and `<?php get_footer(); ?>` snippets in your templates.  They are used to inject additional content into the template, such as third-party analytics code.

Do copy code from the Bootstrap theme included with SiteWriter.

## Common Component Names

To make theme transitions as seamless as possible, try to use the same component names as other themes for equivalent components.

* Page subtitle: `lead`, plain text
* Page content: `content`, rich text
* Page banner (not page title): `banner-title`, plain text
* Content cards: `cardrow-$i`, rich text (where $i is a number between 1 and 3)
* Action buttons above the fold: `banner-btn-$i`, complex component with icon, link, and text (where $i is 1 or 2)
* Contact form submit button: `submit-btn`, complex component with icon and text

Consult the included Bootstrap and Verti themes for example usage of these common components.

## Example Template

Below is a minimal functional `default` template that fulfills the above requirements.

```html
<!DOCTYPE html>
<meta charset=`utf-8`>
<title><?php get_site_name(); ?></title>
<script defer src="<?php get_theme_url(); ?>/fontawesome-all.min.js"></script>
<?php get_header(); ?>
<!-- Navbar -->
<ul>
	<?php get_navigation(); ?>
</ul>
<h1><?php get_page_title(); ?></h1>
<h2>
	<div class="sw-text" data-component="lead">
		<?php get_component("lead"); ?>
	</div>
</h2>
<div class="sw-editable" data-component="content">
	<?php get_page_content(); ?>
</div>
<footer>
	<!-- Social Media -->
	<ul>
        <?php
        $social = get_socialmedia_urls();
        foreach ($social as $s) {
            ?>
            <a class="btn btn-outline-primary m-1" href="<?php echo $s['url']; ?>">
                <i class="<?php echo $s['icon']; ?> fa-fw"></i>
                <span class="sr-only"><?php echo $s['name']; ?></span>
            </a>
            <?php
        }
        ?>
	</ul>
</footer>
```

## Code Snippets

### Plain Text Component

```html
<p class="sw-text" data-component="lead">
  <?php get_component("lead"); ?>
</p>
```

### HTML Component

```html
<div class="sw-editable" data-component="component-name">
  <?php get_component("component-name"); ?>
</div>
```

### Complex Component

```html
<?php
if (!is_complex_empty("banner-btn-2")) {
  $btn = get_complex_component("banner-btn-2", null, ['icon', 'image', 'link', 'text']);
  $icon = $btn['icon'];
  $image = $btn['image'];
  $link = $btn['link'];
  $text = $btn['text'];
  ?>
  <a href="<?php get_url_or_slug($link); ?>" class="sw-complex" data-json="<?php echo get_escaped_json($btn); ?>" data-component="banner-btn-2">
    <i class="<?php echo $icon; ?>"></i>
    <?php echo $text; ?>
    <img src="<?php echo $image; ?>" />
  </a>
<?php } ?>
```

## Theme Functions

### `get_site_name($echo = true)`

Get the name of the website.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_site_url($echo = true)`

Get the URL of the website.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_page_title($echo = true)`

Get the page title.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_page_clean_title($echo = true)`

Get the page title stripped of any HTML.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_page_slug($echo = true)`

Get the page slug for the current page.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_page_clean_url($echo = true, $slug = null)`

Get a valid minimal URL for a page.

 * **Parameters:**
   * `$echo` — `boolean` — default true
   * `$slug` — `string` — page slug, or null for current
 * **Returns:** `string` — 

### `get_page_url($echo = true, $slug = null)`

Get a valid URL for a page.

 * **Parameters:**
   * `$echo` — `boolean` — default true
   * `$slug` — `string` — page slug, or null for current
 * **Returns:** `string` — 

### `get_component($name, $context = null, $echo = true)`

Echoes or returns the content of a component.

 * **Parameters:**
   * `$name` — `string` — component name
   * `$context` — `string` — page slug, or null for current
   * `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `is_component_empty($name, $context = null)`

Check if a component is empty of content.

 * **Parameters:**
   * `$name` — `string` — component name
   * `$context` — `string` — page slug, or null for current
 * **Returns:** `boolean` — 

### `get_complex_component($name, $context = null, $include = [])`

Return the data for a complex component (icon, link, text, image, etc)

 * **Parameters:**
   * `$name` — `string` — component name
   * `$context` — `string` — page slug, or null for current
   * `$include` — `array` — list of properties to include in the output
 * **Returns:** `array` — 

### `is_complex_empty($name, $context = null)`

Check if the specified complex component is empty.

 * **Parameters:**
   * `$name` — `string` — 
   * `$context` — `string` — page slug
 * **Returns:** `boolean` — 

### `get_escaped_json($json, $echo = true)`

Convert a variable into encoded JSON for safe inclusion in an element property.

 * **Parameters:**
   * `$json` — or array to convert to JSON
   * `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_url_or_slug($str, $echo = true)`

Detects if a string is a URL or a page slug, and returns something usable for href

 * **Parameters:**
   * `$str` — `string` — 
   * `$echo` — `boolean` — 
 * **Returns:** `string` — 

### `get_page_content($slug = null)`

Shortcut for get_component("content").

 * **Parameters:** `$slug` — `string` — Get the content for the passed page instead of the current.

### `get_header()`

Echoes invisible page header content.

### `get_footer()`

Echoes invisible page footer content.

### `get_setting($key, $echo = false)`

Return or echo the value of the given site setting key, or an empty string if unset.

 * **Parameters:**
   * `$key` — `string` — 
   * `$echo` — `boolean` — default false
 * **Returns:** `string` — 

### `get_theme_url($echo = true)`

Get the URL path for the theme folder, without trailing slash.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_theme_color_url($echo = true)`

Get the URL base for the selected theme color asset folder, without trailing slash.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_navigation($currentpage = null, $classPrefix = "", $liclass = "", $currentclass = "current", $linkclass = "", $currentlinkclass = "active")`

Get the page navigation as a string containing a series of `<li><a></a></li>` elements.

**Format:**  
Current page:
```html
<li class="$classPrefix$slug $liclass $currentclass">
  <a class="$linkclass $currentlinkclass" href="url">
    Link Text
  </a>
</li>
```
Other pages:
```html
<li class="$classPrefix$slug $liclass">
  <a class="$linkclass" href="url">
    Link Text
  </a>
</li>
```

 * **Parameters:**
   * `$currentpage` — `string` — The page slug to use for context, or null for current.
   * `$classPrefix` — `string` — 
   * `$liclass` — `string` — 
   * `$currentclass` — `string` — default "current"
   * `$linkclass` — `string` — 
   * `$currentlinkclass` — `string` — default "active"

### `output_conditional($content, $var)`

Replace "[[VAR]]" with the contents of $var and echo $content, but only if $var isn't empty.

 * **Parameters:**
   * `$content` — `string` — 
   * `$var` — `string` — 

### `get_fontawesome_js($echo = true)`

Echos or returns a URL for the FontAwesome 5 JavaScript.

 * **Parameters:** `$echo` — `boolean` — default true
 * **Returns:** `string` — 

### `get_socialmedia_urls()`

Returns an array of social media URLs, with FontAwesome icon classes and labels.

 * **Returns:** `array` — `[["icon", "name", "url"]]`